#!/usr/bin/python
# -*- coding: utf-8 -*-

import wave
import struct

# słownik przechowujący miejsca na których znajdują się epoki dla danych samogłosek
periods_dic = dict( [ (a.split()[0], (a.split()[1], a.split()[2])) for a in open('data/periods.txt')] )

def phonetimes(filename='data/czasy.txt'):
	dic = {}
	for line in open(filename):
		list, t = line.split()[:-1], line.split()[-1]
		for g in list:
			dic[g] = float(t)
	return dic

times_dic = phonetimes()	#słownik przechowujący czasy trwania poszczególnych głosek
	
def byte2short(bytelist):
	vals = []
	for i in range(0, len(bytelist), 2):
		vals.append(struct.unpack('h', bytelist[i]+bytelist[i+1])[0])
	return vals

def short2byte(vallist):
	bytes = []
	for v in vallist:
		bytes.append(struct.pack('h', v))
	return ''.join(bytes)


			


def phonetime(phone):
	if phone in times_dic:
		return times_dic[phone]
	else:
		return times_dic['*']
		

def create_wave(filename, bytelist):
	W = wave.open(filename,"w")
	W.setnchannels(1)
	W.setsampwidth(2)
	W.setframerate(44100)
	W.writeframes(bytelist)
	W.close()
	
def get(phone):

	"""zwraca listę bajtów odpowiadających głosce phone"""

	P = wave.open('dzwieki/%s.wav' % phone, 'r')
	framenumber = int(phonetime(phone) * P.getframerate())
	return cut(P.readframes(framenumber))
	
def getA(phone):

	""" zwraca listę bajtów odpowiadających głosce phone, ale zaakcentowanej """
	
	P = freq(louder(get_longer(phone)))
	return cut(P)

def get_longer(phone, mult = 1.5):
	
	""" zwraca listę bajtów odpowiadających głosce phone, ale dłuższą mult-razy """
	
	P = wave.open('dzwieki/%s.wav' % phone, 'r')
	framerate = P.getframerate()
	P.close()
	bytelist = get(phone)
	time = mult * len(bytelist)/2/framerate
	try: (start, end) = int(periods_dic[phone][0]), int(periods_dic[phone][1])
	except: return bytelist

	ntimes = int((framerate*float(time) - len(bytelist)/2 + end - start)/(end - start))
	list = byte2short(bytelist)
	lengthen = list[:start] + list[start:end]*(ntimes if ntimes > 1 else 2) + list[end:]
	
	return short2byte(lengthen[:int(time * framerate)])

	

def louder(bytelist):

	""" podgłaśnia falę reprezentowaną przez bytelist i również zwraca listę bajtów """

	i, List = 0, []
	while i < len(bytelist):
		val = struct.unpack('h', bytelist[i]+bytelist[i+1])[0]*2
		if val >= 2**15:
			val = 2**15 - 1
		if val <= -(2**15):
			val = -2**15 + 1
		List.append(struct.pack('h', val))
		i += 2
	return ''.join(List)

def freq(bytelist, alfa = 1.05):
	
	""" zwraca g(x) = f(alfa*x) """
	
	from math import floor, ceil
	vals = byte2short(bytelist)
	g = vals[:1]
	i = 1
	while ceil(alfa*i) < len(vals):
		xa, xb = int(floor(alfa*i)), int(ceil(alfa*i))
		ya, yb = vals[xa], vals[xb]
		
		g.append((ya+yb)/((xa+xb)/i))
		i += 1
	
	return short2byte(g)


def cutend(bytelist):
	
	""" odcina nagranie od tyłu tak, zeby fala kończyła się na zerze i szła w górę """
	
	i = -2
	up = False
	while True:
		val = struct.unpack('h', bytelist[i]+bytelist[i+1])[0]
		if val > 0:
			up = True
		if val <= 0 and up is True:
			break
		i -= 2
	return bytelist[:i]

def cutbegin(bytelist):
	
	""" odcina nagranie od przodu tak, zeby fala zaczynała od zera i szła w górę """
	
	i = 0
	down = False
	while True:
		val = struct.unpack('h', bytelist[i]+bytelist[i+1])[0]
		if val < 0:
			down = True
		if val >= 0 and down is True:
			break
		i += 2
	return bytelist[i:]
	
def cut(bytelist):
	""" przycina bytelist z przodu i z tyłu (patrz opisy funkcji cutend i cutbegin) """
	return cutbegin(cutend(bytelist))